import React, { useState, useEffect } from 'react';
import { Modal, Button, Table, Form, Input, Select, Space } from 'antd';
import { Greenhouse, GreenhouseInfo, HeatingMode, WateringMode } from '../core/api/data-contracts';
import { Link } from 'react-router-dom';

export function GreenHouseListPage() {
  const [greenhouses, setGreenhouses] = useState<GreenhouseInfo[]>([]);
  const [isModalVisible, setIsModalVisible] = useState(false);
  const [editingGreenhouse, setEditingGreenhouse] = useState<GreenhouseInfo | null>(null);
  const [form] = Form.useForm();


  useEffect(() => {
    fetchGreenhouses();
  }, []);

  const fetchGreenhouses = async () => {
    setGreenhouses([
      {
        id: 1735006258520,
        soilTemperature: 32,
        percentWater: 80,
        autoWateringStatus: Boolean(WateringMode.Value1),
        pumpStatus: Boolean(WateringMode.Value1),
        heatingStatus: Boolean(HeatingMode.Value1)
      },
      {
        id: 1735006258521,
        soilTemperature: 30,
        percentWater: 82,
        autoWateringStatus: Boolean(WateringMode.Value1),
        pumpStatus: Boolean(WateringMode.Value1),
        heatingStatus: Boolean(HeatingMode.Value1)
      },
    ]);
  };

  const handleAddOrEditSubmit = () => {
    form.validateFields().then((values) => {
      if (editingGreenhouse) {
        setGreenhouses(greenhouses.map(g => g.id === editingGreenhouse.id ? { ...g, ...values } : g));
      } else {
        setGreenhouses([...greenhouses, { id: Date.now(), ...values }]);
      }
      setIsModalVisible(false);
      setEditingGreenhouse(null);
      form.resetFields();
    });
  };

  const handleDelete = (id?: number) => {
    setGreenhouses(greenhouses.filter(g => g.id !== id));
  };

  const handleModalOpen = (greenhouse?: GreenhouseInfo) => {
    setEditingGreenhouse(greenhouse || null);
    setIsModalVisible(true);
    if (greenhouse) {
      form.setFieldsValue(greenhouse);
    }
  };
  const columns = [
    {
      title: 'ID',
      dataIndex: 'id',
      key: 'id',
    },
    {
      title: 'Температура',
      dataIndex: 'soilTemperature',
      key: 'soilTemperature',
    },
    {
      title: 'Процент влажности',
      dataIndex: 'percentWater',
      key: 'percentWater',
    },
    {
      title: 'Режим полива',
      dataIndex: 'autoWateringStatus',
      key: 'autoWateringStatus',
      render: (status: boolean) => status ? 'Автоматически' : 'Вручную',
    },
    {
      title: 'Статус вентиля',
      dataIndex: 'pumpStatus',
      key: 'pumpStatus',
      render: (status: boolean) => status ? 'Открыт' : 'Закрыт',
    },
    {
      title: 'Статус нагревателя',
      dataIndex: 'heatingStatus',
      key: 'heatingStatus',
      render: (status: boolean) => status ? 'Включен' : 'Выключен',
    },
    {
  title: 'Действия',
  key: 'action',
  render: (_: any, record: GreenhouseInfo) => (
    <Space>
      <Button onClick={() => startWatering(record.id)}>Начать полив</Button>
      <Button onClick={() => startHeating(record.id)}>Начать нагрев</Button>
      <Button onClick={() => handleDelete(record.id)}>Удалить</Button>
      <Button onClick={() => handleModalOpen(record)}>Редактировать</Button>
      <Link to={`/greenhouses/${record.id}`}>Перейти</Link>
    </Space>
  ),
},
];
return (
  <div className="container mx-auto p-4">
    <h1 className="text-3xl font-bold mb-4">Список теплиц</h1>

    <Button onClick={() => handleModalOpen()} className="mb-4">
      Добавить теплицу
    </Button>

    <Table columns={columns} dataSource={greenhouses} rowKey="id" />

    <Modal
      title={editingGreenhouse ? "Редактирование теплицы" : "Добавление теплицы"}
      visible={isModalVisible}
      onOk={handleAddOrEditSubmit}
      onCancel={() => {
        setIsModalVisible(false);
        form.resetFields();
        setEditingGreenhouse(null);
      }}
    >
      <Form form={form} layout="vertical">
        <Form.Item name="soilTemperature" label="Температура">
          <Input type="number" />
        </Form.Item>
        <Form.Item name="percentWater" label="Процент влажности">
          <Input type="number" />
        </Form.Item>
        <Form.Item name="autoWateringStatus" label="Режим автополива">
          <Select>
            <Select.Option value={WateringMode.Value0}>Отключен</Select.Option>
            <Select.Option value={WateringMode.Value1}>Включен</Select.Option>
          </Select>
        </Form.Item>
        <Form.Item name="pumpStatus" label="Статус вентиля">
          <Select>
            <Select.Option value={false}>Закрыт</Select.Option>
            <Select.Option value={true}>Открыт</Select.Option>
          </Select>
        </Form.Item>
        <Form.Item name="heatingStatus" label="Статус нагревателя">
          <Select>
            <Select.Option value={false}>Отключен</Select.Option>
            <Select.Option value={true}>Включен</Select.Option>
          </Select>
        </Form.Item>
      </Form>
    </Modal>
  </div>
);
};
function startWatering(id: number | undefined): void {
  return;
}

function startHeating(id: number | undefined): void {
  return;
}

